#!/usr/bin/python

"""Functions for finding statistical combinations."""

__author__ = "Joel Burton <joel@joelburton.com>"
__version__ = "1.0"


def combinations(items, n):
    """Return all combinations of items that are n in length.

    @param items: Items to combine
    @type  items: iterable

    @param n: Length of combinations to return
    @type  n: int

    @returns: Iterable of combinations; each combination is a list.

    aabc,3 => aab aac abc abc  (last twice because of different a's)

      >>> list(combinations("aabc", 3))
      [['a', 'a', 'b'], ['a', 'a', 'c'], ['a', 'b', 'c'], ['a', 'b', 'c']]

      >>> list(combinations("abc", 0))
      [[]]
    """

    if n==0: yield []
    else:
        for i in xrange(len(items)):
            for cc in combinations(items[i+1:],n-1):
                yield [items[i]]+cc

def uniqSortedCombinations(items, n):
    """Return all alpha-unique combinations once sorted.

    @param items: Items to combine
    @type  items: iterable

    @param n: Length of combinations to return
    @type  n: int

    @returns: Iterable of unique (once-sorted) combinations; each combination is a tuple.

    aabc,3 => aab aac abc

        >>> list(uniqSortedCombinations("aabc", 3))
        [('a', 'a', 'b'), ('a', 'a', 'c'), ('a', 'b', 'c')]
    """

    seen = {}
    for c in combinations(items, n):
        c = tuple(sorted(c))
        if c in seen: 
            continue
        seen[c] = 1
        yield c

    # more consise:
    # return [list(x) for x in sets.Set(tuple(sorted(x)) for x in a)]





"""User-configurable settings."""

import os

# Allow setting in environmental variables

DICTIONARIES_DIR = os.getenv("DICTIONARIES_DIR")
if not DICTIONARIES_DIR:

    # Store dictionaries in your home directory.

    DICTIONARIES_DIR = os.path.expanduser("~") + "/.anagrammer"

    # Uncomment this for system-wide dictionaries.
    # You may need to make this directory writable to
    # normal users for them to create dictionaries here.
    #
    #DICTIONARIES_DIR = "/usr/local/share/games/anagrammer/"

# Change this to the name of the dictionary you want to
# use by default (otherwise, the first one found is the
# default

DEFAULT_DICT = "default"

MINLENGTH_DEFAULT = 5



import unittest
import doctest
from anagrammer import anagrammer, convert, lookup, combinations

suite = unittest.TestSuite()
for mod in anagrammer, convert, lookup, combinations:
    suite.addTest(doctest.DocTestSuite(mod))
suite.addTest(doctest.DocFileSuite('anagrammer.txt'))
runner = unittest.TextTestRunner(verbosity=9)
runner.run(suite)



#!/usr/bin/python

"""Looks up anagrams for one or more rack(s) of letters.

Finds all anagrams in dictionary. Any letters in
uppercase will be required to be used. Underscores can be
used to represent wildcards or blank tiles. Dictionary can be chosen
among existing dictionaries. Minimum numbers of letters for
resulting words can be chosen."""

__author__ = "Joel Burton <joel@joelburton.com>"
__version__ = "1.0"

import sys
import os.path
import glob
import optparse
import shelve
import textwrap

import convert
import lookup
from config import DICTIONARIES_DIR, MINLENGTH_DEFAULT, DEFAULT_DICT


def anagrammer(words, dictfile, minlength, column=False):
    """Anagram words using dictfile and print results.
    
    @param words: words to anagram
    @type  words: iterable

    @param dictfile: path to shelve dictionary file
    @type  dictfile: string

    @param minlength: minimum # of letters for words
    @type  minlength: int

    @param column: print in single column?
    @type  column: boolean

    @returns: Nothing, just prints

    Tests for this function are in tests/anagrammer.txt.
    """

    worddict = shelve.open(dictfile, "r")

    if not column: print   # create top space

    for word in words:
        found = lookup.lookup(word, minlength, worddict)

        if column:
            if found:
                print "\n".join(found)

        else: 
            print word

            # pretty print, grouped by num letters
            if not found:
               print "\nNone found.\n"
               return

            print
            for wlen in range( len(found[0]), 0, -1 ):
                words = [ w for w in found if len(w) == wlen ]
                if words:
                    print "%s = %s" % (wlen, len(words))
                    print textwrap.fill(
                            " ".join(sorted(words)), 
                            initial_indent="  ", 
                            subsequent_indent="  ")
            print


def start():
    """Interactive start of program."""

    dicts = glob.glob("%s/*.anagram" % DICTIONARIES_DIR)
    dicts = [ os.path.splitext( os.path.basename(d) )[0] for d in dicts ]

    if DEFAULT_DICT in dicts:
        default_dict = DEFAULT_DICT
    else:
        default_dict = dicts[0]

    if dicts:
        parser = optparse.OptionParser(
                usage="%prog [options] rack ...", 
                version="%prog " + __version__, 
                description=__doc__
                )
    else:
        parser = optparse.OptionParser(
                usage="%prog [options]", 
                version="%prog " + __version__, 
                description="""Before using this program to find anagrams, you must
create an anagram dictionary with --convert. Once you have done this,
you can then re-rerun the program to find anagrams."""
                )
    parser.add_option(
            "--convert",
            dest="dictpath",
            help="Convert wordlist (plaintext or gzipped) at DICTPATH into anagram file.",
            type="string",
            )
    if dicts:
        if len(dicts) > 1:
            parser.add_option(
                    "-d", "--dict", 
                    dest="dict", 
                    help="Dictionary to use. Choice of (%s). Defaults to %s." % ( "|".join(dicts), default_dict ),
                    default=default_dict,
                    type="choice",
                    choices=dicts,
                    )
        parser.add_option(
                "-c", "--column",
                dest="column",
                help="Display in simple column (for other programs to read).",
                action="store_true",
                )
        parser.add_option(
                "-l","--min-length",
                dest="minlength",
                help="Minimum number of letters for found words (default: %s)." % MINLENGTH_DEFAULT,
                type="int",
                default=MINLENGTH_DEFAULT,
                )

    (options, args) = parser.parse_args()

    if options.dictpath:
        convert.convert(options.dictpath, DICTIONARIES_DIR)
        sys.exit()

    if len(args)<1:   # no word given
        parser.print_help()
        sys.exit(1)

    word = args[0:]
    dictfile = "%s/%s.anagram" % (DICTIONARIES_DIR, getattr(options, "dict", default_dict))

    anagrammer(word, dictfile, options.minlength, options.column)

if __name__ == "__main__": start()



#!/usr/bin/python

"""Functions for creating anagram dictionaries."""

__author__ = "Joel Burton <joel@joelburton.com>"
__version__ = "1.0"

import os.path
import gzip
import shelve

def alphaword(w):
    """foobar -> abfoor
    
    @param w: word to convert

    @returns: word with letters in alpha order

    For example:

      >>> alphaword("foobar")
      'abfoor'

      >>> alphaword("")
      ''

    """

    return ''.join(sorted(list(w)))



def convert(wordlist, path):
    """Convert a word list into an anagram dictionary.
       
    @param wordlist: path to word list (plaintext or gzipped)
    
    @param path: directory to store resulting file in

    @returns: None, but creates resulting file as side-effect.

    To demonstrate, let's set up a fake word list:

      >>> PATH="/tmp"
      >>> f = open("%s/doctest.lst" % PATH, 'w')
      >>> print >>f, "cat"
      >>> print >>f, "act"
      >>> print >>f, "dog"
      >>> f.close()
      
    Now convert it and test it:

      >>> convert("%s/doctest.lst" % PATH, PATH)

      >>> d = shelve.open("%s/doctest.anagram" % PATH)

      >>> d['act']
      ['cat', 'act']

      >>> d['dgo']
      ['dog']

      >>> d['pig']
      Traceback (most recent call last):
      ...
      KeyError: 'pig'

    And clean up:

      >>> os.unlink("%s/doctest.lst" % PATH)
      >>> os.unlink("%s/doctest.anagram" % PATH)
    """

    if not os.path.isdir(path):
        if os.path.exists(path):
            raise "FileExists", "A file exists at %s, which should be a path for our dictionaries." % path
        os.mkdir(path)

    name = os.path.splitext(os.path.basename(wordlist))[0]
    d = shelve.open("%s/%s.anagram" % (path, name), "n")

    if wordlist.endswith('.gz'):
        opener = gzip.GzipFile
    else:
        opener = file

    for word in opener(wordlist):
        word = word.strip().lower()
        letters = alphaword(word)

        # Shelve doesn't support "x in list" or "list.setdefault" and doesn't
        # notice values mutating, so we have do this very verbosely:

        if d.has_key(letters):
            d[letters] = d[letters] + [word]
        else:
            d[letters] = [word]
    d.close()

